import pgzrun
import random
import sys

# Window Properties
TITLE = 'Goose VS Fruit'
WIDTH = 800
HEIGHT = 800

score = 0
high_score = 0
space_pressed = False

#Game Screen Using Code Outline
game = Actor('game-title')
game.midtop = (375, 0)
game.state = ['title', 'game', 'gameover']
game.current_state = game.state[0]

#Play Goose
player = Actor('goose_normal')
player.pos = (WIDTH // 2, HEIGHT - 100)

#List of Goose Attack
consumes = []
#List of Debris
fruits = []

#Drawing Background
def draw_background():
    for x in range(0, 800, 400):
        for y in range(0, 800, 400):
            screen.blit('background_space',(x, y))
            
def draw_main_menu():
    game.draw()
    screen.blit('press-enter', (300, 500))
    
def draw_game_over():
    screen.blit('game-over', (200, 200))
    screen.blit('press-enter', (300, 500))
    
def draw_consumes():
    global consumes
    for consume in consumes:
        consume.draw()
        
def draw_fruits():
    global fruits
    for fruit in fruits:
        fruit.draw()
        
def draw():
    draw_background()
    if game.current_state == 'title':
        draw_main_menu()
    elif game.current_state == 'game':
        draw_consumes()
        player.draw()
        draw_fruits()
        draw_score()
    elif game.current_state == 'gameover':
        draw_game_over()
    elif game.current_state == 'ending':
        draw_ending()
        
def draw_ending():
    screen.blit('good_job', (200, 200))
    screen.blit('captain_marvel', (200, 300))
    screen.blit('continue', (300, 500))

def draw_score():
    global high_score
    screen.draw.text("Score: {}". format(score), (20, 20), color = "white", fontsize = 30)
    screen.draw.text("High Score: {}". format(high_score), (20, 60), color = "white", fontsize = 30)
    
def check_keys(dt):
    global space_pressed
    if game.current_state == 'game':
        if keyboard.LEFT:
            player.x -= dt * 300
            if player.left < 0:
                player.left = 0
        if keyboard.RIGHT:
            player.x += dt * 300
            if player.right > WIDTH:
                player.right = WIDTH
        if keyboard.UP:
            player.y -= dt * 300
            if player.top < 0:
                player.top = 0
        if keyboard.DOWN:
            player.y += dt * 300
            if player.bottom > HEIGHT:
                player.bottom = HEIGHT
        if keyboard.SPACE and len(consumes) < 10 and not space_pressed:
            space_pressed = True
            spawn_consume()
    
def move_consumes(dt):
    global consumes
    for consume in consumes:
        consume.y -= dt * 400
    
def move_fruits(dt):
    global fruits
    for fruit in fruits:
        fruit.y += dt * 400
        
        
def check_consume_collisions():
    global consumes
    global fruits
    global score
    
    for consume in consumes:
        for fruit in fruits:
            if (
                consume.x < fruit.x + fruit.width
                and consume.x + consume.width > fruit.x
                and consume.y < fruit.y + fruit.height
                and consume.y + consume.height > fruit.y
            ):
                consume.render = False
                fruit.render = False
                sounds.yoshi_tongue.play()
                
                score += fruit.point_value
                print("Score:", score)
                
def check_player_collisions():
    global fruits
    global consumes
    global score
    global high_score
    
    for fruit in fruits:
        if player.colliderect(fruit):
            game.current_state = game.state[2]
            sounds.cat_dying.play()
            clock.unschedule(spawn_fruit)
            fruits = []
            consumes = []
            player.image = 'goose_after'
            clock.schedule_unique(reset_goose_image, 0.2)
            
            print("Game Over. Your Score:", score)
            
            if score > high_score:
                high_score = score
                print("New High Score:", high_score)
                
            score = 0
            
def reset_goose_image():
    player.image = 'goose_normal'
            
def update(dt):
    global score
    check_keys(dt)
    move_consumes(dt)
    move_fruits(dt)
    check_consume_collisions()
    check_player_collisions()
    clean_up()
    
    if score >= 50000:
        game.current_state = 'ending'
        clock.unschedule(spawn_fruit)
    elif game.current_state == 'gameover':
        clock.unschedule(spawn_fruit)
    
def clean_up():
    global consumes
    global fruits
    new_consumes = []
    for consume in consumes:
        if not consume.bottom < 0 and consume.render:
            new_consumes.append(consume)
    consumes = new_consumes
    
    new_fruits = []
    for fruit in fruits:
        if not fruit.top > HEIGHT and fruit.render:
            new_fruits.append(fruit)
    fruits = new_fruits
    
def spawn_consume():
    global consumes
    consume = Actor('goose_eat')
    consume.pos = (player.x, player.y)
    consume.render = True
    consumes.append(consume)
    sounds.yoshi_tongue.play()
    
def spawn_fruit():
    global fruits
    fruit_images = {'apple': 20,
                    'banana': 50,
                    'blackberry': 30,
                    'cherry': 60,
                    'grapes': 80,
                    'orange': 70, 
                    'peach': 90,
                    'pear': 5,
                    'pineapple': 10,
                    'raspberry': 40,
                    'strawberry': 100,
                    'clover': 150,
                    'pumpkin': 200,
                    'pancakes': 250,
                    'candy_cane': 300,
                    'lamp': 500,
                    'table': 1000}
    fruit_image = random.choice(list(fruit_images.keys()))
    fruit = Actor(fruit_image)
    
    xpos = random.randint(fruit.width //2, WIDTH - fruit.width // 2)
    fruit.midbottom = (xpos, 0)
    fruit.render = True
    fruits.append(fruit)
    
    fruit_point_value = fruit_images[fruit_image]
    fruit.point_value = fruit_point_value
    
def on_key_down(key):
    global score
    if game.current_state == 'title' or game.current_state == 'gameover':
        if key == keys.RETURN:
            game.current_state = game.state[1]
            clock.schedule_interval(spawn_fruit, 0.25)
        elif key == keys.SPACE and len(consumes) < 4:
            spawn_consume()
        if key == keys.ESCAPE:
            sys.exit()
    elif game.current_state == 'ending':
        if key == keys.RETURN:
            game.current_state = 'title'
            clock.unschedule(spawn_fruit)
            score = 0
    
def on_key_up(key):
    global space_pressed
    if key == keys.SPACE:
        space_pressed = False  


            
pgzrun.go()